/***************************************************************************
 *   Copyright (C) 2004 by Roberto Raggi                                   *
 *   roberto@kdevelop.org                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GDBPROCESS_H
#define GDBPROCESS_H

#include "miparser.h"

#include <memory>

#include <qprocess.h>
#include <qvaluelist.h>
#include <qpair.h>

/**
@author Roberto Raggi
*/
class GDBProcess : public QProcess
{
    Q_OBJECT
public:
    GDBProcess(bool quit, QObject *parent = 0);
    ~GDBProcess();

    virtual bool start(QStringList *env = 0);

signals:
    void applicationOutput(const QString &text);
    void applicationError(const QString &text);
    void processed(GDBMI::Record *record);

public slots:
    void pushCommand(const QString &command);
    void sync();
    void abort();

private slots:
    void slotReadStdout();
    void slotReadStderr();
    void slotProcessExited();

private:
    void initialize();
    void executeCommand();
    std::auto_ptr<GDBMI::Record> processLine(const QString &text);

private:
    bool exitWhenFinished;
    enum { Initializing, Processing, Accepting } state;
    QValueList<QString> commands;

    MIParser parser;
};

#endif
